import os
import subprocess
import requests
import openai

def generate_backend_code(repo_path):
    # Generate the code using the OpenAI API
    response = openai.Completion.create(
        engine="text-davinci-003",
        prompt="Generate a Node.js backend using Express with the following routes: ...",
        max_tokens=1000
    )

    # Save the generated code to the specified directory
    with open(f"{repo_path}/server.js", "w") as file:
        file.write(response.choices[0].text.strip())

    print(f"Backend code generated in {repo_path}")

def create_git_repo(path):
    os.chdir(path)
    subprocess.run(["git", "init"])
    with open("README.md", "w") as f:
        f.write("# New Project\nGenerated by AI Agent")
    subprocess.run(["git", "add", "."])
    subprocess.run(["git", "commit", "-m", "Initial commit"])
    print(f"Git repository initialized at {path}")

def create_github_repo(repo_name, github_token):
    url = "https://api.github.com/user/repos"
    headers = {"Authorization": f"token {github_token}"}
    data = {"name": repo_name, "private": False}
    response = requests.post(url, headers=headers, json=data)
    
    if response.status_code == 201:
        print(f"Successfully created GitHub repository: {repo_name}")
    else:
        print(f"Failed to create GitHub repository: {response.json()}")

def run_node_server(path):
    os.chdir(path)
    system = os.uname().sysname  # Use platform.system() in Python >= 3.6
    
    if system in ["Linux", "Darwin"]:  # macOS and Linux
        subprocess.run(["npm", "install"])
        subprocess.run(["node", "server.js"])
    elif system == "Windows":
        subprocess.run(["npm", "install"], shell=True)
        subprocess.run(["node", "server.js"], shell=True)

def setup_and_run_backend(project_name, github_token=None):
    home_dir = os.path.expanduser("~")
    core_dir = os.path.join(home_dir, "projects", project_name)
    os.makedirs(core_dir, exist_ok=True)

    generate_backend_code(core_dir)
    create_git_repo(core_dir)

    if github_token:
        create_github_repo(project_name, github_token)
        subprocess.run(["git", "remote", "add", "origin", f"https://github.com/your_username/{project_name}.git"])
        subprocess.run(["git", "push", "-u", "origin", "main"])

    run_node_server(core_dir)

